/**
 * JASMINe
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: jasmine@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.ow2.jasmine.jade.wrapper.jonas4;

import java.io.File;
import java.net.InetAddress;
import java.util.Set;

import org.objectweb.jasmine.configurator.ConfigurationException;
import org.objectweb.jasmine.configurator.Configurator;
import org.objectweb.jasmine.jade.resource.Resource;
import org.objectweb.jasmine.jade.util.DebugResources;
import org.objectweb.jasmine.jade.util.JadeException;
import org.objectweb.jasmine.jade.util.Logger;
import org.objectweb.jasmine.launcher.LaunchException;
import org.objectweb.jasmine.launcher.Launcher;
import org.ow2.jasmine.jade.fractal.api.control.GenericAttributeController;
import org.ow2.jasmine.jade.fractal.api.control.NoSuchAttributeException;
import org.ow2.jasmine.jade.wrapper.jonas4.configurator.JOnASConfigurator;
import org.ow2.jasmine.jade.wrapper.jonas4.launcher.LauncherJonasImpl;

public class JonasRootWrapper implements GenericAttributeController, Resource {

    /**
     *
     */
    String nodeName;
    /**
     *
     */
    String dirLocal;
    /**
     *
     */
    final static String[] attList = { "dirLocal" };
    /**
     *
     */
    Launcher launcher;
    /**
     *
     */
    Configurator configurator;

    // ------------------------------------------------------------------------
    // Constructor
    // ------------------------------------------------------------------------

    public JonasRootWrapper() throws JadeException {

        configurator = new JOnASConfigurator();
        launcher = new LauncherJonasImpl(configurator);

        try {
            nodeName = InetAddress.getLocalHost().getHostName();
        } catch (Exception e) {
            throw new JadeException("cannot read HostName " + e);
        }
    }

    // -------------------------------------------------------------------------
    // Implementation of GenericAttributeController interface
    // -------------------------------------------------------------------------

    /* (non-Javadoc)
     * @see org.ow2.jasmine.jade.fractal.api.control.GenericAttributeController#setAttribute(java.lang.String, java.lang.String)
     */
    public void setAttribute(String name, String value)
            throws NoSuchAttributeException {

        if ("dirLocal".equals(name)) {
            this.dirLocal = value;
            if (!dirLocal.endsWith(File.separator)) {
                dirLocal += File.separator;
            }
        }

        configurator.setProperty(name, value);
    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.jade.fractal.api.control.GenericAttributeController#getAttribute(java.lang.String)
     */
    public String getAttribute(String name) throws NoSuchAttributeException {

        if ("dirLocal".equals(name))
            return dirLocal;

        Object res = configurator.getProperty(name);
        if(res instanceof String){
            return (String) res;
        } else {
            return res.toString();
        }

    }

    /* (non-Javadoc)
     * @see org.ow2.jasmine.jade.fractal.api.control.GenericAttributeController#listFcAtt()
     */
    public String[] listFcAtt() {

        Set<Object> keys = configurator.getProperties().keySet();

        return keys.toArray(new String[0]);
    }

    // ------------------------------------------------------------------------
    // Implementation of the Resource interface
    // ------------------------------------------------------------------------

    /* (non-Javadoc)
     * @see org.objectweb.jasmine.jade.resource.Resource#configure()
     */
    public void configure() throws JadeException {

        Logger.println(DebugResources.on, "[JonasRootWrapper] configure");

        try {
            launcher.configure();
            configurator.configure();
        } catch (ConfigurationException e) {
            throw new JadeException(e);
        } catch (LaunchException e) {
            e.printStackTrace();
        }
    }

    /* (non-Javadoc)
     * @see org.objectweb.jasmine.jade.resource.Resource#loadApp()
     */
    public void loadApp() throws JadeException {
        Logger.println(DebugResources.on, "[JonasRootWrapper] loadApp");
    }

    /* (non-Javadoc)
     * @see org.objectweb.jasmine.jade.resource.Resource#start()
     */
    public void start() throws JadeException {

        Logger.println(DebugResources.on, "[JonasRootWrapper] start");

        try {
            launcher.start();
        } catch (LaunchException e) {
            throw new JadeException(e);
        }
    }

    /* (non-Javadoc)
     * @see org.objectweb.jasmine.jade.resource.Resource#stop()
     */
    public void stop() throws JadeException {

        Logger.println(DebugResources.on, "[JonasRootWrapper] stop");

        try {
            launcher.stop();
        } catch (LaunchException e) {
            throw new JadeException(e);
        }
    }
}
