/**
 * JASMINe
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.ow2.jasmine.jade.wrapper.jonas4.configurator;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.StringTokenizer;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.objectweb.jasmine.configurator.statik.velocity.AbstractVelocityCompositeConfigurator;
import org.objectweb.jasmine.configurator.statik.velocity.VelocityConfigurator;
import org.objectweb.jasmine.configurator.statik.velocity.VelocityConfiguratorImpl;

/**
 * @author Julien Legrand
 *
 */
public class JOnASConfigurator extends AbstractVelocityCompositeConfigurator {

    /**
     *
     */
    Map<String, ConfiguratorProperties> properties = null;

    /**
     *
     */
    private final static String PROPS_FILE_NAME = "configurator.properties";

    /**
     *
     */
    ClassLoader cl = this.getClass().getClassLoader();

    /**
     *
     */
    private static Logger logger = Logger.getLogger(JOnASConfigurator.class
            .getName());

    // ------------------------------------------------------------------------
    // Constructor
    // ------------------------------------------------------------------------

    public JOnASConfigurator() {

        /*
         * construct VelocityContext parent
         */
        super();

        /*
         * construct Configurator from property file
         */
        try {

            parseProperties();

            constructCompositeConfigurator();

        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    // ------------------------------------------------------------------------
    // Private methods
    // ------------------------------------------------------------------------

    private void constructCompositeConfigurator() {

        Set<String> keys = properties.keySet();

        for (String key : keys) {

            ConfiguratorProperties cp = properties.get(key);

            if (cp.isComplete()) {

                VelocityConfigurator vc = new VelocityConfiguratorImpl(this
                        .getContext(), cl, cp.getTemplate(), cp.getFile());
                this.addConfigurator(cp.getPrefix(), vc);

                logger.log(Level.INFO, "Configurator " + cp.toString()
                        + " added.");
            }
        }
    }

    /**
     * @throws IOException
     */
    private void parseProperties() throws IOException {

        if (properties == null) {
            properties = new HashMap<String, ConfiguratorProperties>();
        }

        Properties props = new Properties();
        props.load(cl.getResourceAsStream(PROPS_FILE_NAME));

        Set<Object> keys = props.keySet();
        for (Object key : keys) {

            StringTokenizer tokenizer = new StringTokenizer(key.toString(), ".");

            if (tokenizer.hasMoreTokens()) {

                String propName = tokenizer.nextToken();
                ConfiguratorProperties cp = properties.get(propName);
                if (cp == null) {
                    cp = new ConfiguratorProperties();
                    properties.put(propName, cp);
                }

                if (tokenizer.hasMoreTokens()) {
                    String propType = tokenizer.nextToken();
                    String propValue = props.getProperty(key.toString());
                    if ("template".equals(propType)) {
                        cp.setTemplate(propValue);
                    } else if ("file".equals(propType)) {
                        cp.setFile(propValue);
                    } else if ("prefix".equals(propType)) {
                        cp.setPrefix(propValue);
                    }
                }
            }
        }
    }

    // ------------------------------------------------------------------------
    // Inner Class
    // ------------------------------------------------------------------------

    class ConfiguratorProperties {

        /**
         *
         */
        private String template;

        /**
         *
         */
        private String file;

        /**
         *
         */
        private String prefix;

        // ------------------------------------------------------------------------
        // Constructor
        // ------------------------------------------------------------------------

        public ConfiguratorProperties() {
        }

        /**
         * @param template
         * @param file
         * @param prefix
         */
        public ConfiguratorProperties(String template, String file,
                String prefix) {

            this.template = template;
            this.file = file;
            this.prefix = prefix;
        }

        // ------------------------------------------------------------------------
        // Public methods
        // ------------------------------------------------------------------------

        public boolean isComplete() {

            return !"".equals(template) && !"".equals(file)
                    && !"".equals(prefix) && template != null && file != null
                    && prefix != null;

        }

        // ------------------------------------------------------------------------
        // Getters/Setters
        // ------------------------------------------------------------------------

        public String getTemplate() {
            return template;
        }

        public void setTemplate(String template) {
            this.template = template;
        }

        public String getFile() {
            return file;
        }

        public void setFile(String file) {
            this.file = file;
        }

        public String getPrefix() {
            return prefix;
        }

        public void setPrefix(String prefix) {
            this.prefix = prefix;
        }

        // ------------------------------------------------------------------------
        // Overriden methods
        // ------------------------------------------------------------------------

        @Override
        public String toString() {
            return "[" + prefix + " : " + template + " : " + file + "]";
        }

    }

}
