/**
 * JASMINe
 * Copyright (C) 2006,2007 Bull S.A.S.
 * Contact: jasmine@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.ow2.jasmine.jade.wrapper.jonas4.launcher;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.objectweb.jasmine.configurator.Configurator;
import org.objectweb.jasmine.launcher.DisplayThread;
import org.objectweb.jasmine.launcher.LaunchException;
import org.objectweb.jasmine.launcher.Launcher;

/**
 *
 * TODO Add a abstract launcher Add a abstract checker for output parsing and
 * verify if application is lunched
 *
 * Add exception gestion in start and stop Start must ask to the checker if the
 * application is started if not ask to checker why, if he say throw exception
 * with explanation else send log file, or output...
 *
 * <p>
 * contributor : Julien Legrand
 * </p>
 *
 * @author Yann Davin <yann.davin@gmail.com>
 */

public class LauncherJonasImpl implements Launcher {

    /**
     *
     */
    Configurator configurator;
    /**
     *
     */
    JonasThread jonasThread = null;
    /**
     *
     */
    private final String SCRIPT_FILE_NAME = "jonas.sh";
    /**
     *
     */
    static Logger logger = Logger.getLogger(LauncherJonasImpl.class.getName());
    /**
     *
     */
    private Boolean started = false;

    // ------------------------------------------------------------------------
    // Constructor
    // ------------------------------------------------------------------------

    public LauncherJonasImpl(Configurator configurator) {

        this.configurator = configurator;
        logger.setLevel(Level.INFO);
    }

    // ------------------------------------------------------------------------
    // Implementation of the LauncherIfc interface
    // ------------------------------------------------------------------------

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.launcher.LauncherIfc#start()
     */
    public void start() throws LaunchException {

        if (jonasThread == null)
            jonasThread = new JonasThread();

        jonasThread.start();
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.launcher.Launcher#stop()
     */
    public void stop() throws LaunchException {
        System.out.println("Stop JONAS");
        Runtime r = Runtime.getRuntime();
        try {

            /*
             * Process p = r.exec(getJonasRoot() + "/bin/unix/jonas" + " stop -n " +
             * getInstanceName());
             */
            String command = new String("sh " + getScriptFileName() + " stop");
            Process s = r.exec(command);
            s.waitFor();
            this.setStarted(false);

            if (jonasThread != null) {
                jonasThread.stopMe();
                jonasThread.join();
                jonasThread = null;
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.launcher.LauncherIfc#configure()
     */
    public void configure() throws LaunchException {
        File jonasbase = new File(getJonasBase());

        if (!jonasbase.exists()) {
            try {
                logger.info(jonasbase.getCanonicalFile().toString()
                        + " doesn't exists, so we create it...");
                this.createJonasBase();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    // ------------------------------------------------------------------------
    // Public methods
    // ------------------------------------------------------------------------

    /**
     *
     */
    public void createJonasBase() {

        if (getJonasRoot() == null)
            return;

        String command = new String("sh " + getScriptFileName() + " base");
        Runtime r = Runtime.getRuntime();

        try {
            Process script = r.exec(command);
            script.waitFor();

            InputStream inputstream = script.getInputStream();
            InputStreamReader inputstreamreader = new InputStreamReader(
                    inputstream);
            BufferedReader bufferedreader = new BufferedReader(
                    inputstreamreader);

            // read the ls output

            String line;
            while ((line = bufferedreader.readLine()) != null) {
                System.out.println(line);
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void removeJonasBase() {
        if (getJonasBase() == null)
            return;
        String command = new String("rm -R " + getJonasBase());
        Runtime r = Runtime.getRuntime();
        try {
            Process script = r.exec(command);
            script.waitFor();
        } catch (Exception e) {
            e.printStackTrace();
        }
        ;
    }

    public Boolean isStarted() {
        return started;
    }

    private void setStarted(Boolean started) {
        this.started = started;
    }

    // ------------------------------------------------------------------------
    // Overriden methods
    // ------------------------------------------------------------------------

    protected void finalize() throws Throwable {
        File f = new File(getScriptFileName());
        if (f.exists()) {
            f.delete();
        }
        if (this.isStarted()) {
            this.stop();
        }
        super.finalize();
    }

    // ------------------------------------------------------------------------
    // Private methods
    // ------------------------------------------------------------------------

    private String getInstanceName() {
        return configurator.getProperty("instance-jonas-name").toString();
    }

    private String getJonasBase() {
        return configurator.getProperty("instance-jonas-base").toString();
    }

    private String getJonasRoot() {
        return configurator.getProperty("instance-jonas-root").toString();
    }

    private String getScriptFileName() {
        String installDir = configurator.getProperty("install-dir").toString();
        if (installDir != null) {
            File file = new File(installDir);
            if (file.exists() && file.isDirectory()) {
                if (installDir.endsWith(File.separator)) {
                    return installDir + SCRIPT_FILE_NAME;
                } else {
                    return installDir + File.separator + SCRIPT_FILE_NAME;
                }
            }
        }

        return SCRIPT_FILE_NAME;
    }

    // ------------------------------------------------------------------------
    // Inner class
    // ------------------------------------------------------------------------

    private class JonasThread extends Thread {

        /**
         *
         */
        boolean stop = false;
        /**
         *
         */
        Process script;
        /**
         * to display error stream of jonas
         */
        DisplayThread edt;
        /**
         * to display normal stream of jonas
         */
        DisplayThread ndt;

        // ------------------------------------------------------------------------
        // Implementation of the Runnable interface
        // ------------------------------------------------------------------------

        public void run() {
            if ((getInstanceName() == null) || (getJonasBase() == null)
                    || (getJonasRoot() == null))
                return;

            Runtime r = Runtime.getRuntime();
            try {

                String command = new String("sh " + getScriptFileName()
                        + " start");
                script = r.exec(command);

                InputStream inputstream = script.getInputStream();
                ndt = new DisplayThread(inputstream, getJonasBase());
                ndt.start();

                InputStream errorstream = script.getErrorStream();
                edt = new DisplayThread(errorstream, getJonasBase());
                edt.start();

                script.waitFor();
                setStarted(true);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        // ------------------------------------------------------------------------
        // Public methods
        // ------------------------------------------------------------------------

        public void stopMe() {
            this.stop = true;
            if (edt != null)
                edt.setStopped(true);
            if (ndt != null)
                ndt.setStopped(true);
            if (script != null) {
                script.destroy();
            }
        }
    }
}
